/*
	Copyright 1992 David E. Wexelblat

	Inspiration for the struction of these functions taken from ps.c

	Copyright tranferred to Chris Lewis on the condition that the
	original author's name remain with this code upon any 
	subsequent distribution.

	Copyright 1992 Chris Lewis
		All Rights Reserved

    See the LICENSE file for a full description of restrictions under which
    this software is provided.

    Function:		Provides translation of Ditroff \D commands to
			HP-GL/2 for HP Laserjet III series (PCL5) printers.
*/

#include "defs.h"

#ifdef LJ3
#include "lj.h"
#include <math.h>

#ifndef	lint
static char SCCSid[] =
    "@(#)lj3draw.c: 1.1 Copyright 92/01/23 14:26:29 Chris Lewis";
#endif

#define to_deg(a)	((180.0/M_PI) * (a))
#define XSCALE 		(int)(8.5 * LJRESOLUTION)
#define YSCALE		(int)(11 * LJRESOLUTION)
#define SPLINE_STEPS 	20

#define ENTER_HPGL2	fputs("\033%1B", stdout)
#define EXIT_HPGL2	fputs("\033%0A", stdout)

#ifdef LJ3_7BIT
# define BASE		32
# define END		95
#else
# define BASE		64
# define END		191
#endif

typedef struct {
    int x, y;
} Pt;

encode_coord(x)
int x;
{
    int x1;

    x1 = (x >= 0) ? x << 1 : ((-x) << 1) + 1;
    while (x1 >= BASE) {
	putchar(63 + (x1 % BASE));
	x1 /= BASE;
    }
    putchar(END + x1);
}
/*	Do ditroff drawing command in s from origX,origY.
	If opcode 0 just moveto
 */
lj3Draw(origX, origY, opcode, ctindex, numbers, s)
int origX, origY;		/* Origin for draw command */
int opcode;			/* Draw function */
int ctindex;			/* Number of parameters */
short *numbers; 		/* Parameters */
char *s;			/* Draw command */
{

    register int i, j;
    double a1, a2;
    int sweep;
    int n, x1, y1, x0, y0;
    double w, t1, t2, t3;
    char *command;
    Pt *p;


    /* map absolute beginning position */
    origX = TROFF2LJX(origX);
    origY = TROFF2LJY(origY);

    switch (opcode) {
	case 'f':
	    if (ctindex == 2)
		ctindex = 1;
	    if (!require(1, ctindex, "set fill", s)) {
	    	DBP((D_PIC, "lj3Draw: fill: %d\n", numbers[0]));

		if ((numbers[0] < 0) || (numbers[0] > 1000))
		    fprintf(stderr, "%s: fill %d out of range [0,1000]\n",
			numbers[0]);
		else {
		    ENTER_HPGL2;
	    	    printf("FT10,%d;",(100-(numbers[0]/10)));
		    EXIT_HPGL2;
		}
	    }
	    return;
	case 't':
	    if (ctindex == 2)
		ctindex = 1;
	    if (!require(1, ctindex, "set thick", s)) {
	    	DBP((D_PIC, "lj3Draw: thich: %d\n", numbers[0]));
		/* width 1 = 1mm, troff resolution = 432 dpi */
		ENTER_HPGL2;
	    	printf("\033%%1BLW%.2f;\033%%0A\n", numbers[0]/17.0);
		EXIT_HPGL2;
	    }

	    return;
    }
    printf(XY,origX,origY);
    if (!opcode)
	return;

    DBP((D_PIC, "lj3Draw: (%d,%d): %s\n", origX, origY, s));

    switch (opcode) {
	case 'c':
	    if (ctindex == 2)
		ctindex = 1;
	    /* FALLTHRU */
	case 'e':
	    command = "CI";
	    break;
	case 'C':
	    /* FALLTHRU */
	    if (ctindex == 2)
		ctindex = 1;
	case 'E':
	    command = "WG";
	    break;
	case 'p':
	    command = "EP";
	    break;
	case 'P':
	    command = "FP";
	    break;
    }

    for (i = 0; i < ctindex; i++)
	numbers[i] = numbers[i] * LJRESOLUTION / TROFFRESOLUTION;

    switch (opcode) {
	case 'l':
	    if (require(2, ctindex, "line", s))
		break;
	    DBP((D_PIC, "lj3Draw: line: %d, %d\n", numbers[0], numbers[1]));
	    ENTER_HPGL2;
	    printf("PD;PR%d,%d;", numbers[0], numbers[1]);
	    EXIT_HPGL2;
	    break;
	case 'a':
	    if (require(4, ctindex, "arc", s))
		break;
	    DBP((D_PIC, "lj3Draw: arc: %d, %d, %d, %d\n",
		numbers[0], numbers[1], numbers[2], numbers[3]));
	    a1 = atan2((double)(-numbers[0]), (double)(-numbers[1]));
	    a2 = atan2((double)(numbers[2]), (double)(numbers[3]));
	    sweep = to_deg(a2-a1);
	    if (sweep < 0)
		sweep += 360;
	    sweep = -sweep;	/* HP-GL/2 wants it in the other direction */
	    DBP((D_PIC, "lj3Draw: arc a1=%f a2=%f sweep=%d\n", a1, a2, sweep));
	    ENTER_HPGL2;
	    printf("PD;AR%d,%d,%d;", numbers[0], numbers[1], sweep);
	    EXIT_HPGL2;
	    break;
	case 'C':
	case 'c':
	    if (require(1, ctindex, (opcode=='C')?"filled circle":"circle", s))
		break;
	    DBP((D_PIC, "lj3Draw: %scircle: %d\n", 
		(opcode == 'C' ? "filled " : ""), numbers[0]));
	    /* Use 'CI r;' for circle, 'WG r,0,360;' for filled circle */
	    ENTER_HPGL2;
	    printf("PU;PR%d,0;%s%d%s;", numbers[0]/2, command, numbers[0]/2,
		(opcode == 'C' ? ",0,360" : ""));
	    EXIT_HPGL2;
	    break;
	case 'e':
	case 'E':
	    if (require(2,ctindex,(opcode=='E')?"filled ellipse":"ellipse",s))
		break;
	    DBP((D_PIC, "lj3Draw: %sellipse: %d, %d\n", 
		(opcode == 'E' ? "filled " : ""), numbers[0], numbers[1]));
	    /* Use 'CI r;' for ellipse, 'WG r,0,360;' for filled ellipse */
	    ENTER_HPGL2;
	    printf("PU;PR%d,0;SC0,%d,0,%d;", numbers[0]/2, 
		(int)(XSCALE * ((float)numbers[1]/numbers[0])), YSCALE);
	    printf("%s%d%s;SC0,%d,0,%d", command, numbers[1]/2,
		(opcode == 'E' ? ",0,360" : ""), XSCALE, YSCALE);
	    EXIT_HPGL2;
	    break;
	case 'p':
	case 'P':
	    if (ctindex&1) {
		fprintf(stderr, 
		    "%s: even number of points for %spolygon: %s\n",
		    progname, (opcode == 'P' ? "filled " : ""), s);
		break;
	    }
	    DBP((D_PIC, "lj3Draw: %spolygon:",
		(opcode == 'P' ? "filled " : "")));
#	    ifdef DEBUG
	    for (i=0; i < ctindex; i++)
		DBP((D_PIC, " %d", numbers[i]));
	    DBP((D_PIC, "\n"));
#	    endif /* DEBUG */
	    ENTER_HPGL2;
	    printf("PM0;PD;PR%d,%d",numbers[0],numbers[1]);
	    for (i = 2; i < ctindex; i+=2)
		printf(",%d,%d",numbers[i],numbers[i+1]);
	    printf(";PM2;%s;", command);
	    EXIT_HPGL2;
	    break;
	case '~':
	    if (ctindex < 4) {
		fprintf(stderr, "%s: too few points for spline: %s\n",
		    progname, s);
		break;
	    }
	    DBP((D_PIC, "lj3Draw: spline:"));
#	    ifdef DEBUG
	    for (i=0; i < ctindex; i++)
		DBP((D_PIC, " %d", numbers[i]));
	    DBP((D_PIC, "\n"));
#	    endif /* DEBUG */
	    n = ctindex/2 + 3;
	    p = (Pt *)malloc(sizeof(Pt) * n);
	    p[1].x = origX;
	    p[1].y = origY;
	    for (i = 2; i < n-1; i++) {
		p[i].x = numbers[(i-2)*2]+p[i-1].x;
		p[i].y = numbers[(i-2)*2+1]+p[i-1].y;
	    }
	    p[0] = p[1];
	    p[n-1] = p[n-2];
	    for (i = 0; i < n; i++)
		DBP((D_PIC, "Pt %d: <%d,%d>\n", i, p[i].x, p[i].y));

	    ENTER_HPGL2;
#ifdef LJ3_7BIT
	    fputs("PD;PE7", stdout);
#else
	    fputs("PD;PE", stdout);
#endif
	    x0 = origX;
	    y0 = origY;
	    for (i = 0; i < n-2; i++) {
		for (j = 0; j < SPLINE_STEPS; j++) {
		    w = (double)j / SPLINE_STEPS;
		    t1 = w * w / 2 ;
		    w = w - 0.5;
		    t2 = 0.75 - w * w;
		    w = w - 0.5;
		    t3 = w * w / 2;
		    x1 = t1*p[i+2].x + t2*p[i+1].x + t3*p[i].x + 0.5;
		    y1 = t1*p[i+2].y + t2*p[i+1].y + t3*p[i].y + 0.5;
		    encode_coord(x1-x0);
		    encode_coord(y1-y0);
		    x0 = x1;
		    y0 = y1;
		}
	    }
	    putchar(';');
	    EXIT_HPGL2;
	    free(p);
            break;
	default:
	    fprintf(stderr, "%s: invalid draw code %c (%s)\n",
		progname, opcode, s);
	    break;
    }
}

#endif /* LJ3 */
