/*
	Copyright 1992 Chris Lewis
		All Rights Reserved

    See the LICENSE file for a full description of restrictions under which
    this software is provided.

    Function:		LaserJet driver, drawing code.

			Needs LJII or later (rules)

*/

#include "defs.h"

#if	defined(LJ) && defined(INSPECIAL)
#include "lj.h"

#ifndef	lint
static char SCCSid[] =
    "@(#)ljdraw.c: 1.1 Copyright 92/01/23 14:26:18 Chris Lewis";
#endif

static int ljLine();

/*	Do ditroff drawing command in s from origX,origY.
	If opcode 0 just moveto

	This is hardly complete.  Consider it a skeleton
	for further work.  Before extending it beyond
	horizontal and vertical lines, I'm going to switch
	the whole driver over to 300ths from 720ths.
	Which should eliminate a little jitter, and removes
	a lot of excess work in line drawing.
 */
ljDraw(origX, origY, opcode, ctindex, numbers, saves)
int origX, origY;
int opcode;
char *saves;
int ctindex;
short *numbers; {

    int i, fill = 0;
    static float curfill = 0;
    static int curthick = 1;
    static int warned = 0;

    /* map absolute beginning position. */
    origX = TROFF2LJX(origX);
    origY = TROFF2LJY(origY);

    if (!opcode) {
	printf(XY, origX, origY);
	lastYPos = origY;
	return;
    }

    if (opcode != 'l')
	if (!warned) {
	    fprintf(stderr, "%s: only lines are currently supported\n", progname);
	    warned = 1;
	    return;
	}

    switch (opcode) {
	case 'f':
	    if (ctindex == 2)
		ctindex = 1;
	    if (!require(1, ctindex, "set fill", saves))
		curfill = (double) numbers[0] / 1000;
	    return;
	case 't':
	    if (ctindex == 2)
		ctindex = 1;
	    if (!require(1, ctindex, "set thick", saves))
		curthick = numbers[0];
	    return;
    }


    DBP((D_CAT, "ljDraw: (%d,%d): %s\n", origX, origY, saves));

    switch (opcode) {
	case 'C':
	    if (ctindex == 2)
		ctindex = 1;
	case 'E':
	case 'P':
	    fill = 1;
	    if (opcode != 'P')
		break;
	case 'l':
	case 'a':
	case 'p':
	case '~':
	    for (i = 0; i < ctindex; i++)
		numbers[i] = numbers[i] * LJRESOLUTION / TROFFRESOLUTION;
    }
    switch(opcode) {
	case 'l':
	    if (require(2, ctindex, "line", saves))
		return;
	    DBP((D_CAT, "drawline: %d, %d\n", numbers[0], numbers[1]));
	    ljLine(origX, origY, origX + numbers[0], origY + numbers[1]);
	    break;
#if	0
	case 'a':
	    if (require(4, ctindex, "arc", saves))
		return;
	    for (i = 0; i < 4; i++) {
		emitnums(numbers[i]);
	    }
	    putnl("Arc");
	    break;
	case 'C':
	case 'c':
	    if (require(1, ctindex, "circle", saves))
		return;
	    numbers[1] = numbers[0];
	    ctindex++;
	case 'E':
	case 'e':
	    if (require(2, ctindex, "ellipse", saves))
		return;
	    DBP((D_CAT, "drawellipse: %d, %d\n", numbers[0], numbers[1]));
	    emitnums((int) numbers[0]);
	    emitnums((int) numbers[1]);
	    putnl("Ellipse");
	    break;
	case 'P':
	case 'p':
	    if (ctindex&1) {
		fprintf(stderr, "%s: even number of points for polygon: %s\n",
		    progname, saves);
		break;
	    }
	    for (i = 0; i < ctindex; i += 2) {
		emitnums((int) numbers[i]);
		emitnums((int) numbers[i+1]);
		putnl("rlineto");
	    }
	    putnl(" closepath");
	    break;
	case '~':
	    if (ctindex < 4) {
		fprintf(stderr, "%s: too few points to spline: %s\n",
		    progname, saves);
		break;
	    }
	    emitnums(numbers[0]/2);
	    emitnums(numbers[1]/2);
	    putnl("rlineto");
	    /* NUM/DEN should be between 0 and 1; the closer it is to 1
		the tighter the curve will be to the guiding lines; 2/3
		is the standard value */
#define	NUM	2
#define	DEN	3
	    for (i = 0; i < ctindex - 2; i += 2) {
		emitnums((numbers[i]*NUM)/(2*DEN));
		emitnums((numbers[i + 1]*NUM)/(2*DEN));
		emitnums(numbers[i]/2 + (numbers[i + 2]*(DEN - NUM))/(2*DEN));
		emitnums(numbers[i + 1]/2 + (numbers[i + 3]*(DEN - NUM))/(2*DEN));
		emitnums((numbers[i] - numbers[i]/2) + numbers[i + 2]/2);
		emitnums((numbers[i + 1] - numbers[i + 1]/2) + numbers[i + 3]/2);
		putnl("rcurveto");
	    }
	    emitnums(numbers[ctindex - 2] - numbers[ctindex - 2]/2);
	    emitnums(numbers[ctindex - 1] - numbers[ctindex - 1]/2);
	    putnl("rlineto");
	    break;
#endif
	default:
	    fprintf(stderr, "%s: invalid draw code %c (%s)\n", progname, opcode,
		saves);
	    break;
    }
}

static
linwid() {
    register int a;
    a = lastPoints / 4;
    if (!a)
	a = 1;
    else if (a > 10)
	a = 10;
    return(a);
}
    

static ljPos(x, y)
register int x, y; {
    printf(XY, x, y);
    lastYPos = y;
}

static
ljHLine(x0, y0, x1, y1)
int x0, y0, x1, y1; {
    static int warning = 0;
    if (!warning) {
	fprintf(stderr, "%s: contains non-vertical or horizontal lines.  Ignored\n",
	    progname);
	warning = 1;
    }
}

static
ljVerLine(x0, y0, y1)
register int x0, y0, y1; {
    register int t;
    if (y0 > y1) {
	t = y0;
	y0 = y1;
	y1 = t;
    }
    t = linwid();
    ljPos(x0, y0-t+1);
    printf("\033*c%da%db0P", t, y1-y0+t);
}

static
ljHorLine(x0, y0, x1)
register int x0, y0, x1; {
    register int t = linwid();
    if (x0 > x1) {
	t = x0;
	x0 = x1;
	x1 = t;
    }
    t = linwid();
    ljPos(x0, y0-t+1);
    printf("\033*c%da%db0P", x1-x0+t, t);
}

static
ljLine(x0, y0, x1, y1)
register int x0, y0, x1, y1; {
    if (y0 == y1)
	ljHorLine(x0, y0, x1);
    else if (x0 == x1)
	ljVerLine(x0, y0, y1);
    else
	ljHLine(x0, y0, x1, y1);

}


#endif
