.\"Copyright 1991 by Chris Lewis 2.8 91/07/11
.TH PSXLATE %%MANEXT%% "Psroff %%T2VERSION%%"
.SH NAME
psxlate \- General Postscript page flipper etcetera
.SH SYNOPSIS
.B psxlate
.B "[\-rb2...]"
.BI "[\-p" n "]"
.BI "[\-s" n "]"
.B "[\-d]"
.B "[\-B|E]"
.\"Now why did I put in a -N option?
.\".BI "[\-N" "a,b,c" "]"
.BI "[\-n" "a,a\-b,..." "]"
.BI "<" input file
.BI ">" output file
.SH DESCRIPTION
.B Psxlate
reads a Postscript file that are conformant with the Adobe Document
Structuring Convention standard (DSC), shuffles the pages around according
to the parameters, and outputs the result.
Further, if you have
.B psnup
from comp.sources.misc (volume 16), you can use the postscript files
that come with it to do n-up printing.
``Conformant'' in this context, means files that begin with ``%!PS\-Adobe\-'',
have pages that are prefixed with ``%%Page:'' comments and have a trailer
prefixed by ``%%Trailer:''.
Case is significant.
.PP
.B Psxlate
is ideally used as the filter in your print spooler system.
.PP
.B Psxlate
can be fed any kind of file.
If the file does not have a ``%!'' header, the file is treated as
ASCII, and is automatically converted to Postscript by a special
version of
.BR asc2ps (%%MANEXT%%).
In this case, the output will be portrait, 12 point, 66 lines per page.
Further, any of the options specified to
.BR psxlate ,
such as n-up or pageflipping will work, because
.B asc2ps
generates DSC compliant postscript.
.PP
On the other hand, if the file starts with ``%!'', but not ``%!PS-Adobe-'',
the file is treated as Postscript, but since it can't be trusted to
conform to the DSC, none of the convertions are performed -
.B psxlate
does nothing to the file.
.PP
Unlike most other page flippers which manipulate the Postscript file
in-memory and are thereby will limit the size of the Postscript file,
.B psxlate
does not manipulate the file in-memory,
hence there are fewer restrictions on the size of the file.
The only restrictions are that
.B Psxlate
has to allocate sufficient memory to hold the longest line of Postscript,
plus 2 long integers per page (possibly doubled - see notes) and that
there is enough disk space for an additional copy of the file.
.PP
.B Psxlate
should be compatible with both PS\-Adobe\-1.x and PS\-Adobe\-2.x formatting
conventions, though, it only recognizes the three tokens noted above.
.PP
.B Psxlate
places no restrictions on the characters in a file, and will handle
binary without complaint, provided that the three tokens that
.B psxlate
looks for are immediately preceded by a newline.
However, note that most Postscript printers will not accept 8-bit
characters, some UNIX systems can't send 8-bit characters down
a serial line without grave difficulty, and it sort of defeats the
purpose of Postscript in the first place.
.SH OPTIONS
.PP
The
.B \-d
option enables debugging.
.PP
The
.B \-n
option permits you to select specific pages for printing.
You can specify pages or ranges of pages.
Ie: \-n4,5,20-25
Selects the fourth, fifth and 20th thru 25th pages for printing.
Note that this code does not pay any attention to the contents of
the "%%Page:" comment, and simply selects the pages according to the
order
.I psxlate
sees them.
If the second part of a range is omitted, it means to the end of the document.
If the first part of a range is omitted, it means from the beginning of the document.
.PP
The other options,
.BR r ,
.BR b ,
.BR 2 ,
.B c ,
plus any others locally implemented, denote different shuffle
algorithms which will be applied in-turn to the Postscript and
can be specified more than once.
Specification of none of these options gives no shuffle at all.
.PP
The
.B r
option denotes page reversal - the order of the pages will be
reversed.
This is particularly handy for laser printers that stack face up
(eg: the Canon I Laser engine in older HP Laserjets and Apple Laserwriters)
and without reversal would require you to do a lot of page shuffling.
If used in conjunction with the \-p and \-s options, this will
also generate the n-up pages in the correct order.
.PP
The
.B b
option denotes book order:
.PP
1 2 3 4 5 6 7 8 .... n
.PP
is transformed into:
.PP
n 1 n\-2 3 n\-4 5 ... 2 n\-1 4 n\-3 ...
.PP
This is the order that you would output pages for 2-up printing to
be rebound into a book.
This can be used in conjunction with the
.B \-B
option
to print the first half of the pages, move the pages back into the input
hopper, and then print the second half on the other side of the paper
as a second run of psxlate with the
.B \-E
option on the same input.
You should consult your printer manual to find out the proper orientation.
Up to 3 blank pages are inserted on the end to get a multiple of 4
pages in total.
.PP
The
.B \-c
option is the ``couple swap'' option.
It is similar to reversal, except that it is pairs of pages that
are swapped.
As in:
.PP
1 2 3 4 5 6 ... n
.PP
is converted to:
.PP
n\-1 n n\-3 n\-2 ... 1 2
.PP
This is useful for running after the
.B \-b
option for printers that need reversal, but since you'll be shuffling
the pages anyways, it's probably not all that critical.
A single page may be appended to get the number of pages to be even.
.PP
The
.B \-2
option transforms:
.PP
1 2 3 4 5 6 7 8
.PP
into:
1 3 5 7 ... 2 4 6 8
.PP
suitable for portrait two-sided output.
.SH "NOTES FOR PSNUP (\-p/\-s)"
The
.B Psnup
distribution from comp.sources.misc (volume 16) contains postscript
programs to do n-up printing.
If you have this software,
.B psxlate
will invoke psnup for you if you specify the
.B \-p
or
.B \-s
options.
.PP
The value of the
.BI \-p n
option indicates how many document pages (or "spots") should be
placed on a page.
.I N
is constrained to be 2, 4, 8 or 16.
.PP
The
.BI \-s n
option indicates where the first spot should be placed.
In non-reversed output
.BI \-s 0
indicates that the first spot will be on the upper left corner.
.BI \-s 1
indicates that the first spot will be in the "next" position.
If you specify
.B \-r
to psxlate, these positions will be reversed.
.SH "NOTES FOR PSROFF"
If you have a ``backwards printing'' printer, you normally have
an invocation of ``psxlate \-r'' in the
.B psrofflib
lparg option for the printer.
If you use a ``\-t'' option to collect the output of
.BR psroff,
there won't be an invocation of
.B psxlate
so the pages will be in sequential ascending order, and you
can run
.B psxlate
on that.
To print two sided on a reversing printer, you would type:
.sp
.nf
psroff .... \-t files > /tmp/saveit
psxlate \-2B < /tmp/saveit | <your printer spooler>
change paper around
psxlate \-r2E < /tmp/saveit | <your printer spooler>
.fi
.PP
On a non-reversing printer, the
.B \-r
option wouldn't be necessary.
.SH FILES
.if t .ta 2.5i
.if n .ta 3.5i
%%LIBDIR%%/nup.pro	Psnup postscript prolog
.br
%%LIBDIR%%/nup.epi	Psnup postscript epilog
.SH "SEE ALSO"
troff2ps(%%MANEXT%%), psroff(%%MANEXT%%), asc2ps(%%MANEXT%%),
and Adobe's Postscript manuals (in particular, the Red Book).
And psnup(1) if you have it.
.SH "NOTES"
New shuffle algorithms are relatively easy to add.
The source ``o_.*'' routines show two different approaches
to implementing new shuffles.
The ``o_book'' routine demonstrates a situation where a new set
of page descriptors has to be allocated because the shuffle algorithm
is too complex to easily do in-place.
The ``o_reverse'' and ``o_dupswap'' routines demonstrate simpler
shuffle algorithms that can be done in-place.
The psxlate.c source code has more detailed instructions on how to
add shuffle functions.
If you write a new one, please let me know.
.PP
You can use
.B psxlate
on things other than Postscript if you arrange to have the three
tokens mentioned at the beginning inserted at the appropriate places
in the file.
If they'll cause problems when you use the file, you can always run the
file through
.B sed
to delete them.
.SH AUTHOR
Written by Chris Lewis
